@@ -8,35 +8,35 @@ import (
 	"github.com/tidwall/gjson"
 	"github.com/v2rayA/v2rayA/db"
 	"log"
+	"sort"
 	"strings"
 )
 
 type Configure struct {
-	Servers                    []*ServerRaw       `json:"servers"`
-	Subscriptions              []*SubscriptionRaw `json:"subscriptions"`
-	ConnectedServer            *Which             `json:"connectedServer"` //冗余一个信息，方便查找
-	AdditionalConnectedServers []Which            `json:"additionalConnectedServers"`
-	Setting                    *Setting           `json:"setting"`
-	Accounts                   map[string]string  `json:"accounts"`
-	Ports                      Ports              `json:"ports"`
-	PortWhiteList              PortWhiteList      `json:"portWhiteList"`
-	InternalDnsList            *string            `json:"internalDnsList"`
-	ExternalDnsList            *string            `json:"externalDnsList"`
-	RoutingA                   *string            `json:"routingA"`
+	Servers          []*ServerRaw       `json:"servers"`
+	Subscriptions    []*SubscriptionRaw `json:"subscriptions"`
+	ConnectedServers []*Which           `json:"connectedServers"`
+	Setting          *Setting           `json:"setting"`
+	Accounts         map[string]string  `json:"accounts"`
+	Ports            Ports              `json:"ports"`
+	PortWhiteList    PortWhiteList      `json:"portWhiteList"`
+	InternalDnsList  *string            `json:"internalDnsList"`
+	ExternalDnsList  *string            `json:"externalDnsList"`
+	RoutingA         *string            `json:"routingA"`
 }
 
 func New() *Configure {
 	return &Configure{
-		Servers:                    make([]*ServerRaw, 0),
-		Subscriptions:              make([]*SubscriptionRaw, 0),
-		ConnectedServer:            nil,
-		AdditionalConnectedServers: nil,
-		Setting:                    NewSetting(),
-		Accounts:                   map[string]string{},
+		Servers:          make([]*ServerRaw, 0),
+		Subscriptions:    make([]*SubscriptionRaw, 0),
+		ConnectedServers: make([]*Which, 0),
+		Setting:          NewSetting(),
+		Accounts:         map[string]string{},
 		Ports: Ports{
 			Socks5:      20170,
 			Http:        20171,
 			HttpWithPac: 20172,
+			VlessGrpc:   0,
 		},
 		PortWhiteList:   PortWhiteList{},
 		InternalDnsList: nil,
@@ -98,14 +98,9 @@ func SetConfigure(cfg *Configure) error {
 	if err := SetPortWhiteList(&cfg.PortWhiteList); err != nil {
 		return err
 	}
-	if err := SetConnect(cfg.ConnectedServer); err != nil {
+	if err := OverwriteConnects(NewWhiches(cfg.ConnectedServers)); err != nil {
 		return err
 	}
-	for _, w := range cfg.AdditionalConnectedServers {
-		if err := SetConnect(&w); err != nil {
-			return err
-		}
-	}
 	if err := SetSetting(cfg.Setting); err != nil {
 		return err
 	}
@@ -198,7 +193,7 @@ func GetSubscriptions() []SubscriptionRaw {
 }
 func GetSubscription(index int) *SubscriptionRaw {
 	s := new(SubscriptionRaw)
-	err := db.ListGet("touch", "subscriptions", index, &s)
+	err := db.ListGet("touch", "subscriptions", index, s)
 	if err != nil {
 		return nil
 	}
@@ -206,8 +201,8 @@ func GetSubscription(index int) *SubscriptionRaw {
 }
 func GetSettingNotNil() *Setting {
 	r := new(Setting)
-	_ = db.Get("system", "setting", &r)
-	if r == nil {
+	err := db.Get("system", "setting", r)
+	if err != nil {
 		r = NewSetting()
 		_ = db.Set("system", "setting", r)
 	}
@@ -219,22 +214,26 @@ func GetSettingNotNil() *Setting {
 	}
 	return r
 }
-func GetPorts() *Ports {
-	r := new(Ports)
-	err := db.Get("system", "ports", &r)
-	if err != nil {
-		return nil
+func GetPortsNotNil() *Ports {
+	p := new(Ports)
+	_ = db.Get("system", "ports", &p)
+	if p == nil {
+		p = new(Ports)
+		p.Socks5 = 20170
+		p.Http = 20171
+		p.HttpWithPac = 20172
+		p.VlessGrpc = 0
 	}
-	return r
+	return p
 }
 func GetPortWhiteListNotNil() *PortWhiteList {
 	r := new(PortWhiteList)
-	_ = db.Get("system", "portWhiteList", &r)
+	_ = db.Get("system", "portWhiteList", r)
 	return r
 }
 func GetExternalDnsListNotNil() (list []string) {
 	r := new(string)
-	_ = db.Get("system", "externalDnsList", &r)
+	_ = db.Get("system", "externalDnsList", r)
 	list = strings.Split(strings.TrimSpace(*r), "\n")
 	if len(list) == 1 && list[0] == "" {
 		return []string{}
@@ -243,7 +242,7 @@ func GetExternalDnsListNotNil() (list []string) {
 }
 func GetInternalDnsListNotNil() (list []string) {
 	r := new(string)
-	_ = db.Get("system", "internalDnsList", &r)
+	_ = db.Get("system", "internalDnsList", r)
 	if len(strings.TrimSpace(*r)) == 0 {
 		*r = `119.29.29.29 -> direct
 https://doh.alidns.com/dns-query -> direct
@@ -257,7 +256,7 @@ tcp://dns.opendns.com:5353 -> proxy`
 }
 func GetCustomPacNotNil() *CustomPac {
 	r := new(CustomPac)
-	_ = db.Get("system", "customPac", &r)
+	_ = db.Get("system", "customPac", r)
 	if r.DefaultProxyMode == "" {
 		r = &CustomPac{
 			DefaultProxyMode: DefaultProxyMode,
@@ -273,35 +272,32 @@ func GetRoutingA() (r string) {
 	}
 	return
 }
-func GetConnectedServers() (wts []*Which) {
+func GetConnectedServers() (wts *Whiches) {
 	outbounds := GetOutbounds()
 	for _, outbound := range outbounds {
-		w := GetConnectedServer(outbound)
+		w := GetConnectedServersByOutbound(outbound)
 		if w != nil {
-			wts = append(wts, w)
+			if wts == nil {
+				wts = new(Whiches)
+			}
+			wts.Extend(*w)
 		}
 	}
-	if len(wts) == 0 {
+	if wts != nil && wts.Len() == 0 {
 		wts = nil
 	}
 	return wts
 }
-func GetConnectedServer(outbound string) *Which {
-	r := new(Which)
-	if outbound == "" || outbound == "proxy" {
-		if err := db.Get("touch", "connectedServer", &r); err != nil {
-			return nil
-		}
-		if r != nil {
-			r.Outbound = "proxy"
-		}
-		return r
-	} else {
-		if err := db.Get(fmt.Sprintf("outbound.%v", outbound), "connectedServer", &r); err != nil {
-			return nil
-		}
-		return r
+func GetConnectedServersByOutbound(outbound string) *Whiches {
+	r := new(Whiches)
+	if outbound == "" {
+		outbound = "proxy"
 	}
+	bucket := fmt.Sprintf("outbound.%v", outbound)
+	if err := db.Get(bucket, "connectedServers", r); err != nil {
+		return nil
+	}
+	return r
 }
 
 func GetLenSubscriptions() int {
@@ -325,29 +321,69 @@ func GetLenServers() int {
 	}
 	return l
 }
-
-func ClearConnected(outbound string) error {
-	if outbound == "" || outbound == "proxy" {
-		return db.Set("touch", "connectedServer", nil)
-	} else {
-		return db.Set(fmt.Sprintf("outbound.%v", outbound), "connectedServer", nil)
+func ClearConnects(outbound string) error {
+	if outbound == "" {
+		outbound = "proxy"
 	}
+	return db.Set(fmt.Sprintf("outbound.%v", outbound), "connectedServers", nil)
+}
+func AddConnect(wt Which) (err error) {
+	if wt.Outbound == "" {
+		wt.Outbound = "proxy"
+	}
+	bucket := fmt.Sprintf("outbound.%v", wt.Outbound)
+	var wcs Whiches
+	_ = db.Get(bucket, "connectedServers", &wcs)
+	for _, v := range wcs.Get() {
+		if v.EqualTo(wt) {
+			return nil
+		}
+	}
+	wcs.Add(wt)
+	return db.Set(bucket, "connectedServers", wcs)
 }
 
-func SetConnect(wt *Which) (err error) {
-	if wt == nil || wt.Outbound == "" || wt.Outbound == "proxy" {
-		return db.Set("touch", "connectedServer", wt)
-	} else {
-		return db.Set(fmt.Sprintf("outbound.%v", wt.Outbound), "connectedServer", wt)
+// OverwriteConnects will replace each outbounds contained in given ws with whiches in the ws
+func OverwriteConnects(ws *Whiches) (err error) {
+	outWs := make(map[string][]*Which)
+	for _, w := range ws.Get() {
+		outWs[w.Outbound] = append(outWs[w.Outbound], w)
 	}
+	for out, ws := range outWs {
+		whiches := new(Whiches)
+		whiches.Touches = ws
+		bucket := fmt.Sprintf("outbound.%v", out)
+		if err := db.Set(bucket, "connectedServers", whiches); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
+func RemoveConnect(wt Which) (err error) {
+	if wt.Outbound == "" {
+		wt.Outbound = "proxy"
+	}
+	bucket := fmt.Sprintf("outbound.%v", wt.Outbound)
+	var wcs Whiches
+	_ = db.Get(bucket, "connectedServers", &wcs)
+	for i, v := range wcs.Touches {
+		if v.EqualTo(wt) {
+			wcs.Touches = append(wcs.Touches[:i], wcs.Touches[i+1:]...)
+			return db.Set(bucket, "connectedServers", wcs)
+		}
+	}
+	return fmt.Errorf("given server cannot be found in database")
 }
 
 func GetOutbounds() (outbounds []string) {
-	outbounds = append(outbounds, "proxy")
+	// keep order
 	members, _ := db.StringSetGetAll("outbounds", "names")
 	for _, m := range members {
 		outbounds = append(outbounds, m)
 	}
+	sort.Strings(outbounds)
+	outbounds = append([]string{"proxy"}, outbounds...)
 	return
 }
 
@@ -382,7 +418,28 @@ func GetPasswordOfAccount(username string) (pwd string, err error) {
 	return
 }
 
+func GetAccounts() (accounts [][2]string, err error) {
+	unames, err := db.GetBucketKeys("accounts")
+	if err != nil {
+		return nil, err
+	}
+	for _, uname := range unames {
+		var passwd string
+		err = db.Get("accounts", uname, &passwd)
+		accounts = append(accounts, [2]string{uname, passwd})
+	}
+	return accounts, nil
+}
+
 func HasAnyAccounts() bool {
 	l, err := db.GetBucketLen("accounts")
 	return err == nil && l > 0
 }
+
+func SetRunning(running bool) (err error) {
+	return db.Set("system", "running", running)
+}
+func GetRunning() (running bool) {
+	_ = db.Get("system", "running", &running)
+	return running
+}
