@@ -1,7 +1,6 @@
 package global
 
 import (
-	"os"
 	"time"
 )
 
@@ -9,7 +8,6 @@ var (
 	Version                  = "debug"
 	FoundNew                 = false
 	RemoteVersion            = ""
-	V2RayPID                 *os.Process
 	TickerUpdateGFWList      *time.Ticker
 	TickerUpdateSubscription *time.Ticker
 )
