@@ -2,8 +2,8 @@ package service
 
 import (
 	"github.com/v2rayA/v2rayA/common"
-	"github.com/v2rayA/v2rayA/common/jwt"
 	"github.com/v2rayA/v2rayA/db/configure"
+	"github.com/v2rayA/v2rayA/server/router/jwt"
 	"time"
 )
 
