@@ -5,12 +5,14 @@ import (
 	"github.com/v2rayA/v2rayA/common/httpClient"
 	"github.com/v2rayA/v2rayA/common/netTools/netstat"
 	"github.com/v2rayA/v2rayA/common/netTools/ports"
+	"github.com/v2rayA/v2rayA/common/resolv"
 	"github.com/v2rayA/v2rayA/core/specialMode"
 	"github.com/v2rayA/v2rayA/core/v2ray"
 	"github.com/v2rayA/v2rayA/core/vmessInfo"
 	"github.com/v2rayA/v2rayA/db/configure"
 	"github.com/v2rayA/v2rayA/plugin"
 	"log"
+	"net"
 	"net/http"
 	"strconv"
 	"strings"
@@ -18,6 +20,8 @@ import (
 	"time"
 )
 
+const HttpTestURL = "http://www.msftconnecttest.com/connecttest.txt"
+
 func Ping(which []*configure.Which, timeout time.Duration) (_ []*configure.Which, err error) {
 	var whiches = configure.NewWhiches(which)
 	//对要Ping的which去重
@@ -50,6 +54,42 @@ func Ping(which []*configure.Which, timeout time.Duration) (_ []*configure.Which
 	return which, nil
 }
 
+func addHosts(tmpl *v2ray.Template, vms []vmessInfo.VmessInfo) {
+	tmpl.DNS = new(v2ray.DNS)
+	tmpl.DNS.Hosts = make(v2ray.Hosts)
+	const concurrency = 5
+	var mu sync.Mutex
+	var limit = make(chan struct{}, concurrency)
+	var wg = sync.WaitGroup{}
+	for _, v := range vms {
+		if net.ParseIP(v.Add) == nil {
+			wg.Add(1)
+			go func(addr string) {
+				limit <- struct{}{}
+				defer func() {
+					wg.Done()
+					<-limit
+				}()
+				ips, err := resolv.LookupHost(addr)
+				if err != nil {
+					return
+				}
+				if len(ips) > 0 {
+					ips = v2ray.FilterIPs(ips)
+					mu.Lock()
+					if v2ray.CheckHostsListSupported() == nil {
+						tmpl.DNS.Hosts[addr] = ips
+					} else {
+						tmpl.DNS.Hosts[addr] = ips[0]
+					}
+					mu.Unlock()
+				}
+			}(v.Add)
+		}
+	}
+	wg.Wait()
+}
+
 func TestHttpLatency(which []*configure.Which, timeout time.Duration, maxParallel int, showLog bool) ([]*configure.Which, error) {
 	specialMode.StopDNSSupervisor()
 	var whiches = configure.NewWhiches(which)
@@ -143,16 +183,23 @@ func TestHttpLatency(which []*configure.Which, timeout time.Duration, maxParalle
 			plugin.GlobalPlugins.Add("outbound"+strconv.Itoa(localPort), plu)
 		}
 	}
+	tmpl.Routing.DomainStrategy = "AsIs"
+	addHosts(&tmpl, vms)
+
 	err = v2ray.WriteV2rayConfig(tmpl.ToConfigBytes())
 	if err != nil {
 		return nil, err
 	}
 
-	if occupied, port, pname := tmpl.CheckInboundPortsOccupied(); occupied {
-		return nil, newError("Port ", port, " is occupied by ", pname)
+	if err = tmpl.CheckInboundPortsOccupied(); err != nil {
+		return nil, newError(err)
 	}
-	err = v2ray.RestartV2rayService()
+
+	process, err := v2ray.RestartV2rayService(false)
 	if err != nil {
+		if process != nil {
+			_ = process.Kill()
+		}
 		return nil, err
 	}
 	//limit the concurrency
@@ -176,14 +223,15 @@ func TestHttpLatency(which []*configure.Which, timeout time.Duration, maxParalle
 		}(i)
 	}
 	wg.Wait()
+	if process != nil {
+		_ = process.Kill()
+		v2ray.SetCoreProcess(nil)
+	}
 	if v2rayRunning && configure.GetConnectedServers() != nil {
 		err = v2ray.UpdateV2RayConfig()
 		if err != nil {
-			return which, newError("failed to restart v2ray-core, please connect a server")
+			return which, fmt.Errorf("cannot restart v2ray-core: %w", err)
 		}
-	} else {
-		// no connected servers or v2ray was not running
-		_ = v2ray.StopV2rayService()
 	}
 	if err := configure.NewWhiches(which).SaveLatencies(); err != nil {
 		return nil, fmt.Errorf("failed to save the latency test result: %v", err)
@@ -203,7 +251,7 @@ func httpLatency(which *configure.Which, port string, timeout time.Duration) {
 	c.CheckRedirect = func(req *http.Request, via []*http.Request) error {
 		return http.ErrUseLastResponse
 	}
-	req, _ := http.NewRequest("GET", "http://www.msftconnecttest.com/connecttest.txt", nil)
+	req, _ := http.NewRequest("GET", HttpTestURL, nil)
 	//req, _ := http.NewRequest("GET", "http://www.gstatic.com/generate_204", nil)
 	req.Header.Set("Accept", "*/*")
 	req.Header.Set("Cache-Control", "no-cache")
